/*
	keyboard.dm
	keyboard library
	A simple keyboard library that can be included in all of your projects

	Written by FIREking

	How to use:
	-fill the keys_list with the keys you want available to access in your game
	-use the following code to activate key control for a mob!

	mob/Login()
		..()
		src.client.add_keys()
		src.client.set_focus(src)

	/////////////////////////////////////////
	reacting to key presses
	/////////////////////////////////////////

	mob/key_down(k, client/c)
		if(k == "north")
			world << "You pressed the up arrow!"

	/////////////////////////////////////////
	make something repeat while holding down a key
	two ways:
	/////////////////////////////////////////

	mob/key_repeat(k, client/c)
		if(k == "north")
			world << "you are holding north"
			//this will go at the speed of the users keyboard repeat delay

	/////////////////////////////////////////

	mob/key_down(k, client/c)
		if(k == "north")
			spawn start_north()
	mob/proc/start_north()
		while(src.client.keys["north"]) //while key is held down
			//do stuff
			sleep(world.tick_lag) //how long to wait before next repeat

	/////////////////////////////////////////
	stop repeating
	/////////////////////////////////////////

	mob/key_down(k, client/c)
		if(k == "north")
			c.keys[k] = 0 //turn the key off even if user is still pressing it
			world << "this will not repeat"

	/////////////////////////////////////////
	detecting a double key press
	/////////////////////////////////////////

	mob/var/tmp/d_timer = 0
	mob/key_down(k, client/c)
		if(k == "d")
			if(world.time - d_timer <= 2)
				world << "dash"
				d_timer = 0
			else
				d_timer = world.time
				world << "walk"

	/////////////////////////////////////////
	changing focus
	/////////////////////////////////////////

	gui_element/menu/key_down(k, client/c)
		c << "you're pressing the [k] key!"
		if(k == "escape")
			c.return_focus()
			del src
	mob/key_down(k, client/c)
		if(k == "escape")
			open_menu()
	mob/proc/open_menu()
		var/gui_element/menu/m = new(src.client)
		src.client.set_focus(m)

	/////////////////////////////////////////
	do something only if two keys are pressed
	/////////////////////////////////////////
	mob/proc/key_down(k, client/c)
		if(k == "s" && c.keys["shift"]) //shift and s are held down
			world << "shift s!"
		else if(k == "s")
			world << "just s!"

	/////////////////////////////////////////
	stop reading keys
	/////////////////////////////////////////

	client.lock_input()

	/////////////////////////////////////////
	clearn input manually (empty key buffers)
	/////////////////////////////////////////

	client.clear_input()

*/

datum/proc/key_down(k, client/c)
datum/proc/key_up(k, client/c)
datum/proc/key_repeat(k, client/c)
datum/proc/focus_lost(client/c)
datum/proc/focus_gained(client/c)

client/proc/key_down(k, client/c)
client/proc/key_up(k, client/c)
client/proc/key_repeat(k, client/c)
client/proc/focus_lost(client/c)
client/proc/focus_gained(client/c)

var/list/keys_list = list(\
	"a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z",
	"1", "2", "3", "4", "5", "6", "7", "8", "9", "0",
	"space",
	"return", //enter
	"escape",
	"tab",
	"shift",
	"alt",
	"ctrl", //control
	"north", //up
	"south", //down
	"east", //right
	"west" //left
)

client
	var/tmp
		focus = null
		last_focus = null
		input_locked = 0
		list/keys = list()

	North()
	South()
	East()
	West()
	Northeast()
	Northwest()
	Southeast()
	Southwest()

	verb/KeyDown(k as text)
		set hidden = 1
		set instant = 1
		if(input_locked) return
		keys[k] = 1
		if(focus) if(hascall(focus, "key_down")) focus:key_down(k, src)

	verb/KeyUp(k as text)
		set hidden = 1
		set instant = 1
		keys[k] = 0
		if(input_locked) return
		if(focus) if(hascall(focus, "key_up")) focus:key_up(k, src)

	verb/KeyRepeat(k as text)
		set hidden = 1
		set instant = 1
		if(input_locked) return
		if(focus) if(hascall(focus, "key_repeat")) focus:key_repeat(k, src)

	proc/add_key(k)
		winset(src, "macro[k]Down", "parent=macro;name=[k];command=KeyDown+[k]")
		winset(src, "macro[k]Up", "parent=macro;name=[k]+UP;command=KeyUp+[k]")
		winset(src, "macro[k]Repeat", "parent=macro;name=[k]+REP;command=KeyRepeat+[k]")
		keys[k] = 0

	proc/add_keys()
		for(var/k in keys_list)
			add_key(k)

	proc/lock_input()
		input_locked = 1

	proc/unlock_input()
		input_locked = 0

	proc/clear_input()
		unlock_input()
		for(var/v in keys)
			keys[v] = 0

	proc/set_focus(x)
		clear_input()
		last_focus = focus
		if(hascall(focus, "focus_lost")) focus:focus_lost(src)
		focus = x
		if(hascall(focus, "focus_gained")) focus:focus_gained(src)

	proc/return_focus()
		set_focus(last_focus)